/*********************************************************

  Author: FieldingBlue
  Date:   3-Jul-2013
  Version: 1.0
  
  The bathroom door keeps being left open during the winter and it
  allows all the heat to escape. This project monitors the state
  of a small reed switch and signals with a small buzzer.
  
  When the door is opened initially, a short beep is produced to 
  remind the operator that the door should be closed.
  When the door has been open for 15 seconds there is another short beep.
  When the door has been open for 30 seconds there is another longer beep.
  When the door has been open for 60 seconds there are two beeps.
  At 90 seconds there are three beeps.
  When the door has been open for 120 seconds there are a series of beeps
  that repeat every 30 seconds. When the door has been left open for 5 minutes
  the unit gives up and waits for the door to be closed. 
  
  
  *********************************************************/



//********************************************************

// Uncomment the #define below to enable DEBUG mode
//#define DEBUG


// This firmware was initially developed for an AtTiny13 and for some reason
// HIGH and LOW don't work as they do for the regular versions of Arduino. So
// some redundent definitions are included here to make different platforms compile
// correctly.
#ifndef HIGH
#define HIGH 1
#endif


#ifndef LOW
#define LOW  0
#endif


#define SENSE_PIN   2
#define BUZZER_PIN  7

#define DEBOUNCE_COUNT 5
#define SENSE_DELAY   50  // (ms)
#define CYCLE_RATE    20  // 1000ms/sense_delay

#define SENSE_ACTIVE_STATE HIGH


#define SHORT_BUZZ 100
#define LONG_BUZZ  250



//********************************************************
long int time0;
long int time1;
int sensecount;
int opentime;
int buzztime;





//********************************************************
void setup(){
  pinMode(SENSE_PIN, INPUT);
  pinMode(BUZZER_PIN, OUTPUT);
  
  #ifdef DEBUG
  Serial.begin(115200);
  Serial.println("Door minder, 3 July 2013, FieldingBlue\n");
  #endif
  
  time0 = millis();
  sensecount = 0;
  opentime = 0;
  buzztime = 0;
}





//********************************************************
void loop(){
  
  time1 = millis();
  if( time1 < time0 ){
    // deal with timer overflow
    time0 = time1;
    
  }else{
    if(time1 - time0 > SENSE_DELAY){
      if(digitalRead(SENSE_PIN) == SENSE_ACTIVE_STATE){
        sensecount ++;
        if(sensecount >= DEBOUNCE_COUNT){
          sensecount = DEBOUNCE_COUNT;
          opentime++;
          
          if(opentime == 1){
            buzz(1, SHORT_BUZZ);
          }else if(opentime == 15 * CYCLE_RATE){
            buzz(1, SHORT_BUZZ);
          }else if(opentime == 30 * CYCLE_RATE){
            buzz(1, SHORT_BUZZ);
          }else if(opentime == 60 * CYCLE_RATE){
            buzz(1, LONG_BUZZ);
          }else if(opentime == 90 * CYCLE_RATE){
            buzz(1, LONG_BUZZ);
          }else if(opentime == 120 * CYCLE_RATE){
            buzz(2, LONG_BUZZ);
          }else if(opentime == 180 * CYCLE_RATE){
            buzz(2, LONG_BUZZ);
          }else if(opentime == 240 * CYCLE_RATE){
            buzz(3, LONG_BUZZ);
          }else if(opentime == 270 * CYCLE_RATE){
            buzz(3, LONG_BUZZ);
          }else if(opentime == 300 * CYCLE_RATE){
            buzz(4, LONG_BUZZ);
          }else if(opentime == 360 * CYCLE_RATE){
            buzz(4, LONG_BUZZ);
          }else if(opentime == 420 * CYCLE_RATE){
            buzz(5, LONG_BUZZ);
          }else if(opentime >= 480 * CYCLE_RATE){
            opentime = 500 * CYCLE_RATE;
          }
        }
      }else{
        sensecount = 0;
        opentime = 0;
      }
      time0 = time1;
    }
  }
}



//********************************************************
void buzz(int repeat, int time){
  int i;
  int j;
  
  #ifdef DEBUG
  Serial.print("buzz\t");
  Serial.print(repeat);
  Serial.print(" * ");
  Serial.println(time);
  #endif
  
  for(j=0; j<repeat; j++){
    for(i=0; i<time; i++){
      digitalWrite(BUZZER_PIN, HIGH);
      delayMicroseconds(500);
      digitalWrite(BUZZER_PIN, LOW);
      delayMicroseconds(500);
    }
    if(repeat>0){
      delay(time);
    }
  }
}
  
